Package: manipulateWidget
Type: Package
Title: Add Even More Interactivity to Interactive Charts
Version: 0.11.1
Authors@R: c(
    person("Veronique", "Bachelier", email = "veronique.bachelier@rte-france.com", role = c("aut", "cre")),
    person("Jalal-Edine", "ZAWAM", role = "aut"),
    person("Francois", "Guillem", role = "aut"),
    person("RTE", role = "cph"),
    person("JJ", "Allaire", role = "ctb"),
	person("Marion", "Praz", email="mnpraz@gmail.com", role = "ctb", comment = "New user interface"),
	person("Benoit", "Thieurmel", role = "ctb", email = "benoit.thieurmel@datastorm.fr"),
	person(given = "Titouan", family = "Robert", email = "titouan.robert@datastorm.fr", role = "ctb"),
	person("Duncan", "Murdoch", email = "murdoch.duncan@gmail.com", role = "ctb")
    )
Description: Like package 'manipulate' does for static graphics, this package
    helps to easily add controls like sliders, pickers, checkboxes, etc. that 
    can be used to modify the input data or the parameters of an interactive 
    chart created with package 'htmlwidgets'.
URL: https://github.com/rte-antares-rpackage/manipulateWidget
License: GPL (>= 2) | file LICENSE
Depends: R (>= 2.10)
Imports: shiny (>= 1.0.3), miniUI, htmltools, htmlwidgets, knitr,
        methods, tools, base64enc, grDevices, codetools, webshot,
        shinyjs
Suggests: dygraphs, leaflet, plotly, xts, rmarkdown, testthat, covr
LazyData: TRUE
RoxygenNote: 7.1.2
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2021-10-05 08:27:14 UTC; BenoitThieurmel
Author: Veronique Bachelier [aut, cre],
  Jalal-Edine ZAWAM [aut],
  Francois Guillem [aut],
  RTE [cph],
  JJ Allaire [ctb],
  Marion Praz [ctb] (New user interface),
  Benoit Thieurmel [ctb],
  Titouan Robert [ctb],
  Duncan Murdoch [ctb]
Maintainer: Veronique Bachelier <veronique.bachelier@rte-france.com>
Repository: CRAN
Date/Publication: 2021-10-05 08:50:09 UTC
Built: R 4.4.0; ; 2024-04-30 14:04:32 UTC; unix
