# Generated by roxygen2: do not edit by hand

S3method(summary,MWController)
export(combineWidgets)
export(combineWidgetsOutput)
export(compareOptions)
export(knit_print.MWController)
export(manipulateWidget)
export(mwCheckbox)
export(mwCheckboxGroup)
export(mwDate)
export(mwDateRange)
export(mwGroup)
export(mwModule)
export(mwModuleUI)
export(mwNumeric)
export(mwPassword)
export(mwRadio)
export(mwSelect)
export(mwSelectize)
export(mwSharedValue)
export(mwSlider)
export(mwText)
export(renderCombineWidgets)
export(staticImage)
export(staticPlot)
exportClasses(MWController)
import(webshot)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(htmltools,tagGetAttribute)
importFrom(htmlwidgets,getDependency)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setRefClass)
importFrom(miniUI,gadgetTitleBar)
importFrom(miniUI,miniContentPanel)
importFrom(miniUI,miniPage)
importFrom(miniUI,miniTabPanel)
importFrom(miniUI,miniTabstripPanel)
importFrom(shiny,fillPage)
importFrom(shiny,fillRow)
importFrom(shiny,icon)
importFrom(shiny,isolate)
importFrom(shiny,observe)
importFrom(shiny,observeEvent)
importFrom(shiny,reactive)
importFrom(shiny,tagAppendChild)
importFrom(shiny,tagAppendChildren)
importFrom(shiny,tags)
importFrom(stats,runif)
importFrom(utils,getFromNamespace)
